/**
 * Create a directory module index.
 * @function apemanmodule
 * @param {string} dirname - Directory path which contains locale messages.
 * @param {object} [options] - Optional settings.
 * @returns {object} modules
 */

"use strict";

const argx = require('argx'),
    stringcase = require('stringcase'),
    print = require('./print'),
    _datasource = require('./_datasource');

/** @lends apemanmodule */
function apemanmodule(dirname, options) {
    let args = argx(arguments);
    options = args.pop('object') || {};
    dirname = args.shift('string');

    let datasource = _datasource(dirname, {
        pascal: options.pascal
    });

    function modules(name) {
        let valid = modules.has(name);
        if (!valid) {
            let guess = modules.guess(name);
            let err = `Unknown module: "${name}".`;
            if (guess) {
                err = `${err} ( Did you mean "${guess}"? )`;
            }
            throw new Error(err);
        }
        return datasource[name];
    }

    modules.__proto__ = Object.assign(new Function(), {
        /**
         * Module data source.
         */
        datasource: datasource,
        /**
         * Check if has a module with name.
         * @param {string} name - Name of module.
         * @returns {*}
         */
        has(name) {
            return datasource.hasOwnProperty(name)
        },
        /**
         * Guess with name.
         * @param {string} name
         * @returns {*}
         */
        guess(name) {
            let converters = [stringcase.pascalcase, stringcase.camelcase];
            for (let convert of converters) {
                let convertedName = convert(name);
                let hit = modules.has(convertedName);
                if (hit) {
                    return convertedName;
                }
            }
            return null;
        },
        /**
         * Print modules
         * @param {string} [namespace] - Namespace to print.
         */
        print(namespace) {
            print(modules, namespace);
        }
    });

    Object.keys(datasource).forEach((key) => {
        Object.defineProperty(modules, key, {
            get: () => {
                return datasource[key];
            },
            configurable: true,
            enumerable: true
        })
    });

    return modules;
}

module.exports = apemanmodule;